@@ -6,6 +6,44 @@ REGEX_PATTERNS = {
     'parameter_key_value' : r"^([a-zA-Z][a-zA-Z0-9_.-/]*)=(.*)$",
 }
 
+CLI_COMMANDS_HELP = {
+    'parameter': {
+        'add': """Add a parameter to a context, or update a parameter If it is already existing in the context.\n
+                ** Tips: 1) If the parameter is inherited from the parent contexts, it will be overriden in the context. 2) Multiple parameters may be added at once using the '--input' option.\n
+                KEY: The identifier of the parameter to be added. It may also be provided using the '--key' option.\n
+                VALUE: The value for the parameter. If the parameter is already existing in the context, the value will be updated to the new one. It may also be provided using the '--value' option.\n
+                """,
+        'list': """Return the list of owned/inherited parameters in a context.\n
+                ** Tips: 1) To limit the list to the owned/overriden parameters only, use the '--uninherited' option. This will return only the context specific parameters.
+                """,
+        'get': """Return the current value of a parameter in a context.\n
+                ** Tips: 1) The parameter may be inherited from the parent contexts or owned by the given context.\n
+                KEY: The identifier of the parameter. It may also be provided using the '--key' option.\n
+                """,
+        'delete': """Delete a parameter from a context.\n
+                ** Tips: 1) The inherited parameters cannot be deleted. In other words, the context must be the owner of the parameter or a not found error will be returned. 2) Multiple parameters may be deleted at once using the '--input' option.\n         
+                KEY: The identifier of the parameter to be deleted. It may also be provided using the '--key' option.\n
+                """,                
+    },
+    'secret': {
+        'add': """Add a secret to a context, or update a secret If it is already existing in the context.\n
+                ** Tips: 1) If the secret is inherited from the parent contexts, it will be overriden in the context. 2) Multiple secrets may be added at once using the '--input' option.\n
+                KEY: The identifier of the secret to be added. It may also be provided using the '--key' option.\n
+                VALUE: The value for the secret. If the secret is already existing in the context, the value will be updated to the new one. It may also be provided using the '--value' option.\n
+                """,
+        'list': """Return the list of owned/inherited secrets in a context.\n
+                ** Tips: 1) To limit the list to the owned/overriden secrets only, use the '--uninherited' option. This will return only the context specific secrets.
+                """,
+        'get': """Return the current value of a secret in a context.\n
+                ** Tips: 1) The secret may be inherited from the parent contexts or owned by the given context.\n
+                KEY: The identifier of the secret. It may also be provided using the '--key' option.\n
+                """,
+        'delete': """Delete a secret from a context.\n
+                ** Tips: 1) The inherited secrets cannot be deleted. In other words, the context must be the owner of the secret or a not found error will be returned. 2) Multiple secrets may be deleted at once using the '--input' option.\n         
+                KEY: The identifier of the secret to be deleted. It may also be provided using the '--key' option.\n
+                """,                
+    },
+}
 CLI_COMMANDS_SHORT_HELP = {
     'version': "Display versions.",
     'parameter': {
@@ -48,36 +86,41 @@ CLI_COMMANDS_SHORT_HELP = {
 }
 CLI_PARAMETERS_HELP = {
     'common': {
-        'working_dir': "Path to the directory where the DSO application configuration resides. Current working directory will be used by default.",
-        'verbosity' : "Logging verbosity: 0 for criticals, 1 for errors, 2 for warnings, 3 for information, 4 for debug, 5 for everything.",
-        'stage' : f"Identifier of the stage, which is combination of a name and an optional number as in <name>[/<number>], and it must conform to '{REGEX_PATTERNS['stage']}'. If no <number> is specefied, the default environment (0) in the given stage will be used.",
-        'input' : "Path to a local file inputing the parameters. Use '-' to read from pipe/stdin.",
-        'format' : "Data format",
-        'config': "Comma separated list of key:value pairs to override DSO configurations.",
+        'working_dir': "Path to a (local) directory where the DSO application configuration resides. By default, the current working directory will be used if the option is not provided.",
+        'verbosity' : "Specify the logging verbosity, where 0 is for logging critical fatal errors only, 1 also logs error messages, 2 also logs warnings, 3 also logs information messages, 4 also logs debug messages, and finally 5 logs everything.",
+        'stage' : "Target a specific stage using the stage identifier, which is combination of a name and an optional number, where name must conform to ^([a-zA-Z][a-zA-Z0-9]+)$ regex expression. If no /<number> is specified, the default environment (/0) in the given stage will be targeted.",
+        'input' : "Path to a (local) file defining the input data. Use '-' to read from the shell pipe or stdin. Use '--format' to specify the format if needed.",
+        'format': "Specify the format of the output or the input if mixed with the '--input' option.",
+        'config': "Comma separated list of key/value pairs to temporarily override the current DSO application configurations. It takes effect only while executing the command and does not have any lasting effect on the DSO application configuration or subsequent command executions.",
+        'query': "Customize output using JMESPath query language.",
+        'query_all': "Include all the available fields in the ouput.",
     },
     'parameter': {
-        'key': "The key of the parameter",
-        'value': "The value for the parameter",
-        'show_values': "Whether or not show the values of parameters.",
-        'uninherited': "Select only parameters which are specific to the gievn context, i.e. not inherited from the parent contexts."
+        'key': "The key of the parameter. See KEY argument for more details.",
+        'value': "The value for the parameter. See VALUE argument for more details.",
+        'query_values': "Include parameter values in the output.",
+        'uninherited': "Select only parameters which are specific to the gievn context, i.e. not inherited from the parent contexts.",
+
     },
     'secret': {
         'key': "The key of the secret",
         'value': "The value for the secret",
-        'show_values': "Whether or not show the values of parameters",
+        'query_values': "Include secret values in the output.",
         'uninherited': "Select only secrets which are specific to the gievn context, i.e. not inherited from the parent contexts."
     },
     'template': {
         'type': "Type of the template. Use 'resource' for templates needed at the provision time when provisioning resources required by the application to run such as SQS queus, SNS topics, and CI/CD piplines.\nUse 'package' for templates needed at the build time when generating a package.\nUse 'release' for templates needed at the deploy time when generating a release." ,
         'key': "The key of the template",
         'limit': "Limit templates to be rendered.",
-        'render_path': "Path (relative to the root of the DSO application) where rendered template will be placed at. Use '**' and/or '*' to add the template dir name and base name to the render path.",
-        'show_render_path': "Show render path",
+        'render_path': "Path (relative to the root of the DSO application) where rendered template will be placed at.",
+        'query_render_path': "Include the template render paths in the output.",
+        'input' : "Path to a local file containing the template content.",
+        'recursive' : "Add files recursively.",
     },
     'config': {
         'key': "The key of the configuration",
         'value': 'Value for the configuration key',
-        'input' : "Path to a local (yaml) file inputing the configuration. Use '-' to read from pipe/stdin.",
+        'input' : "Path to a local (yaml) file inputing the configuration. Use '-' to read from the shell pipe or stdin.",
         'local': "Select only the local DSO configurations, i.e. existing in the working directory.",
         'global': "Select only the global DSO configurations, i.e. user-wide configuration.",
         'init_local': "Explicitly override inherited configurations locally. If mixed with '-i' / '--input' option, it will casue the local configuration to be merged with the provided input configuration.",
@@ -88,21 +131,12 @@ CLI_PARAMETERS_HELP = {
 
 }
 
-# CLI_MESSAGES = {
-#     'InvalidArgumentValue': "Invalid argument value: '{0}' is invalid for '{1}'. Must conform to '{2}'",
-#     'AtleastOneofTwoArgumentsNeeded': "At least one of '{0}' or '{1}' must be provided.",
-#     'ArgumentsMutalExclusive': "'{0}' is mutually exclusive with '{1}'.",
-# }
 
-
-
-MESSAGES = {
+CLI_MESSAGES = {
     'InvalidKey': "'{0}' is an invalid key. Must conform to '{1}'",
     'ParameterNotFound': "Parameter '{0}' not found in the given context.",
     'SecretNotFound': "Secret '{0}' not found in the given context.",
-    # 'ParameterNotFoundScope': "'{0}' not found as a parameter, but found a scope.",
     'InvalidStage': "'{0}' is not a valid stage name. Valid form is <string>[/number], where it must conform to '{1}'.",
-    'DSOConfigNotFound': 'DSO application configuration not found.',
     'ContextNotFound': "Context '{0}' not found.",
     'PatternNotMatched': "'{0}' is invalid. Must conform to '{1}'",
     'InvalidParameterKeyValuePair': "'{0}' is an invalid parameter key/value pair. Must conform to '^([a-zA-Z][a-zA-Z0-9_.-/]*)=(.*)$'",
@@ -111,6 +145,7 @@ MESSAGES = {
     'MissingOption': "Missing option {0}.",
     'MissingArgument': "Missing argument {0}.",
     'ArgumentsMutualExclusive': "Either {0} or {1} must be provided, but not both.",
+    'ArgumentsMutualInclusive': "{0} is needed when {1} is provided.",
     'TemplateNotFound': "Template '{0}' not found.",
     'InvalidTemplateKey': "'{0}' is an invalid template key. Must conform to '{1}'",
     'ContextNotFoundListingInherited': "Context '{0}' not found, listing inherited parameters if any.",
@@ -128,14 +163,19 @@ MESSAGES = {
     'RenderingTemplate': "Rendering '{0}'...",
     'OptionMutualInclusive': "Option {0} needed when {1} is provided.",
     'InvalidDSOConfigurationFile': "'{0}' is not a valid DSO configuration file.",
+    'ProviderNotSet': "{0} provider has not been set.",
+    'InvalidDSOConfigOverrides': "Invalid DSO configuration overrides. Must conform to '<key>=<value>, ...'",
+    'DSOConfigutrationOverriden': "DSO configuration '{0}' overriden to '{1}'.",
     'DSOConfigNewer': "Application is configured to use a newer version of dso, expected '{0}', got '{1}'.",
     'DSOConfigOlder': "Application is configured to use an older version of dso, expected '{0}', got '{1}'.",
-    'ProviderNotSet': "{0} provider has not been set.",
-    'InvalidConfigOverrides': "Invalid DSO configuration overrides. Must conform to '<key>=<value>, ...'",
-    'DSOConfigutrationOverriden': "DSO configuration '{0}' overriden to '{1}'.",
+    'DSOConfigurationNotFound': 'DSO configuration not found.',
     'NoDSOConfigFound': "No DSO configuration found in the working directory.",
     'EnteredSecretValuesNotMatched': "Entered values for the secret did not macth.",
     'InvalidRenderPath': "'{0}' is not a valid render path.",
     'InvalidRenderPathExistingDir': "'{0}' is not a valid render path because it is an existing directory.",
     'InvalidRenderPathAbs': "'{0}' is not a valid render path becasue render path must be relative to the root of the DSO application.",
+    'QueryOptionCompatibleFormats': "Option '-q' / '--query' can be used only with 'json'/'yaml' output formats.",
+    'QueryAllOptionNonCompatibleFormats': "Option '-a'/ '--query-all' cannot be used with 'shell' output format.",
+    'MissingCSVField': "Field '{0}' is missing.", 
+    'FileNotFound': "File '{0}' not found.",
 }
