@@ -1,6 +1,6 @@
 class DSOException(Exception):
     
-    def __init__(self, message):
+    def __init__(self, message=''):
         self.__message = message
         super().__init__(message)
 
