@@ -27,6 +27,7 @@ class ParametersClass():
         return provider.list(project, application, stage, uninherited)
 
     def add(self, stage, key, value):
+        self.validate_key(key)
         project = Config.project
         application = Config.application
         provider = ProviderManager.ParameterProvider()
@@ -35,6 +36,7 @@ class ParametersClass():
         return provider.add(project, application, stage, key, value)
 
     def get(self, stage, key):
+        # self.validate_key(key)
         project = Config.project
         application = Config.application
         provider = ProviderManager.ParameterProvider()
@@ -43,6 +45,7 @@ class ParametersClass():
         return provider.get(project, application, stage, key)
 
     def delete(self, stage, key):
+        # self.validate_key(key)
         project = Config.project
         application = Config.application
         provider = ProviderManager.ParameterProvider()
