@@ -11,7 +11,7 @@ from .providers import StoreProvider, ProviderManager
 from .parameters import Parameters
 from .secrets import Secrets
 from .logger import Logger
-from .utils import clean_directory, merge_dicts
+from .utils import clean_directory, merge_dicts, deflatten_dict
 from .exceptions import DSOException
 from .stages import Stages
 
@@ -22,7 +22,7 @@ class TemplateProvider(StoreProvider):
     def add(self, key, content, render_path):
         raise NotImplementedError()
 
-    def delete(self, key):
+    def delete(self, key, starts_with):
         raise NotImplementedError()
 
     def get(self, key):
@@ -42,48 +42,52 @@ class TemplatesClass():
         return provider.list(project, application)
 
     def add(self, key, content, render_path):
+        self.validate_key(key)
         project = Config.project
         application = Config.application
         provider = ProviderManager.TemplateProvider()
         if render_path:
             # if '..' in render_path:
-            #     raise DSOException(MESSAGES['InvalidRenderPath'].format(render_path))
-            if '**' in render_path:
-                render_path = render_path.replace('**', os.path.dirname(key))
-            if '*' in render_path:
-                render_path = render_path.replace('*', os.path.basename(key))
-            if not re.match('^[A-Za-z0-9._/$-]+$', render_path):
-                raise DSOException(MESSAGES['InvalidRenderPath'].format(render_path))
+            #     raise DSOException(CLI_MESSAGES['InvalidRenderPath'].format(render_path))
             if render_path == '.' or render_path == './':
                 render_path = os.path.join(self.default_render_path, key)
+            else:
+                if '**' in render_path:
+                    render_path = render_path.replace('**', os.path.dirname(key))
+                if '*' in render_path:
+                    render_path = render_path.replace('*', os.path.basename(key))
+            if not re.match('^[A-Za-z0-9._/$-]+$', render_path):
+                raise DSOException(CLI_MESSAGES['InvalidRenderPath'].format(render_path))
             if os.path.isdir(render_path):
-                raise DSOException(MESSAGES['InvalidRenderPathExistingDir'].format(render_path))
+                raise DSOException(CLI_MESSAGES['InvalidRenderPathExistingDir'].format(render_path))
             if os.path.isabs(render_path):
                 Logger.warn(f"Render path {render_path} is not releative to the application root.")
         try:
             Logger.info(f"Start adding template: project={project}, application={application}, key={key}, render_path={render_path}")
             result = provider.add(project, application, key, content)
         finally:
-            if render_path == os.path.join(self.default_render_path, key):
+            if os.path.abspath(render_path) == os.path.abspath(os.path.join(self.default_render_path, key)):
                 Config.unregister_template_custom_render_path(key)
             else:
                 Config.register_template_custom_render_path(key, render_path)
         return result
 
     def get(self, key):
+        # self.validate_key(key)
         project = Config.project
         application = Config.application
         provider = ProviderManager.TemplateProvider()
         Logger.info(f"Start getting template: project={project}, application={application}, key={key}")
         return provider.get(project, application, key)
 
-    def delete(self, key):
+    def delete(self, key, recursive):
+        # self.validate_key(key)
         project = Config.project
         application = Config.application
         provider = ProviderManager.TemplateProvider()
         Logger.info(f"Start deleting template: project={project}, application={application}, key={key}")
         try:
-            result = provider.delete(project, application, key)
+            result = provider.delete(project, application, key, recursive)
         finally:
             Config.unregister_template_custom_render_path(key)
         return result
@@ -93,7 +97,6 @@ class TemplatesClass():
         Logger.info("Start validating template key...")
         return provider.validate_key(key)
 
-
     def render(self, stage, limit=''):
         project = Config.project
         application = Config.application
@@ -102,23 +105,25 @@ class TemplatesClass():
         if Stages.is_default(stage):
             Logger.warn("No stage has been provided for rendering templates, using 'default'.")
 
-        Logger.info(MESSAGES['LoadingParameters'])
-        params = Parameters.list(stage, uninherited=False)
-        
-        Logger.info(MESSAGES['LoadingSecrets'])
+        Logger.info(CLI_MESSAGES['LoadingParameters'])
+        parameters = Parameters.list(stage, uninherited=False)
+
+        merged = deflatten_dict({x['Key']: x['Value'] for x in parameters['Parameters']})
+
+        Logger.info(CLI_MESSAGES['LoadingSecrets'])
         secrets = Secrets.list(stage, uninherited=False, decrypt=True)
 
-        Logger.info(MESSAGES['MerginParameters'])
-        params = merge_dicts(secrets, params)
-        
+        Logger.info(CLI_MESSAGES['MerginParameters'])
+        merge_dicts(deflatten_dict({x['Key']: x['Value'] for x in secrets['Secrets']}), merged)
+
         provider = ProviderManager.TemplateProvider()
-        Logger.info(MESSAGES['LoadingTemplates'])
+        Logger.info(CLI_MESSAGES['LoadingTemplates'])
         templates = provider.list(project, application)
         renderPaths = Config.get_template_render_path()
 
         jinja_env = jinja2.Environment(undefined=jinja2.StrictUndefined)
 
-        Logger.info(MESSAGES['RenderingTemplates'])
+        Logger.info(CLI_MESSAGES['RenderingTemplates'])
         rendered = []
         for item in templates:
             key = item['Key']
@@ -136,7 +141,7 @@ class TemplatesClass():
             # if len(undeclaredParams) > 0:
             #     Logger.warn(f"Undecalared parameter(s) found:\n{set(undeclaredParams)}")
             try:
-                renderedContent = template.render(params)
+                renderedContent = template.render(merged)
             except:
                 Logger.error(f"Failed to render template: {key}")
                 raise
