@@ -2,6 +2,7 @@
 from .logger import Logger
 from .exceptions import DSOException
 
+
 def clean_directory(path):
     for path in Path(path).glob("**/*"):
         if path.is_file():
@@ -31,7 +32,6 @@ def merge_dicts(source, destination):
 
 ###--------------------------------------------------------------------------------------------
 
-
 def flatten_dict(input_node: dict, prefixed_key = '', delimiter = '.', output_dict: dict = {}):
     if isinstance(input_node, dict):
         for key, val in input_node.items():
@@ -46,6 +46,14 @@ def flatten_dict(input_node: dict, prefixed_key = '', delimiter = '.', output_di
 
 ###--------------------------------------------------------------------------------------------
 
+def deflatten_dict(input: dict, delimiter = '.'):
+    result = {}
+    for key, value in input.items():
+        set_dict_value(result, key.split(delimiter), value, overwrite_parent=True, overwrite_children=True)
+    return result
+
+###--------------------------------------------------------------------------------------------
+
 def get_dict_item(dic, keys, create=True):
     for i in range(0, len(keys)):
         key = keys[i]
@@ -107,4 +115,27 @@ def del_dict_empty_item(dic, keys):
     if len(item) == 0:
         item = get_dict_item(dic, keys[:-1])
         item.pop(keys[-1])
-        del_dict_empty_item(dic, keys[:-1])
\ No newline at end of file
+        del_dict_empty_item(dic, keys[:-1])
+
+
+###--------------------------------------------------------------------------------------------
+
+def is_file_binary(filename):
+    """ 
+    Return true if the given filename appears to be binary.
+    File is considered to be binary if it contains a NULL byte.
+    FIXME: This approach incorrectly reports UTF-16 as binary.
+    """
+    with open(filename, 'rb') as f:
+        for block in f:
+            if b'\0' in block:
+                return True
+    return False
+
+###--------------------------------------------------------------------------------------------
+
+def print_list(items):
+    if len(items):
+        for item in items:
+            print(item, flush=True)
+
